-- Script by Johan Thorngren a.k.a. Trinisica
-- http://www.trinisica.com

-- START
plugin Geometry AreaLight 
name:"Area Light"
classId:#(0x4076ac03, 0xce718b1d)
extends:PlanGomtrieXX
category:"Trinisica"
version: 0.08
(

parameters AreaLight rollout:AreaLight_roll (
	lightColor			type:#color 	default:[255,255,255] animatable:true 	 ui:lightColor
	visibleOn			type:#boolean	default:true		animatable:false	ui:visibleOn
	areaIntensity		type:#float 	default:3 			animatable:false	ui:areaIntensity
	lightsHotspot		type:#integer	default:90			animatable:false 	ui:lightsHotspot
	lightsFalloff		type:#integer 	default:110 		animatable:false 	ui:lightsFalloff
	areaShadows			type:#boolean	default:true		animatable:false	ui:areaShadows
	areaShadowsRTd		type:#boolean	default:false		animatable:false	ui:areaShadowsRTd
	areaOvershoot		type:#boolean	default:false		animatable:false	ui:areaOvershoot
	lightsSpecular		type:#boolean	default:true		animatable:false	ui:lightsSpecular
	arealightOn			type:#boolean	default:true		animatable:false	ui:arealightOn
	areaShadowMapSize	type:#integer	default:256			animatable:false 	ui:areaShadowMapSize
	areaShadowMapSample	type:#float		default:4			animatable:false 	ui:areaShadowMapSample
	areaShadowMapBias	type:#float		default:1			animatable:false 	ui:areaShadowMapBias
	)

parameters AreaLight rollout:AreaLightFalloff_roll (
	lightsDecayType		type:#integer	default:1			animatable:false 	ui:lightsDecayType
	lightsDecayStart	type:#float		default:40			animatable:true 	ui:lightsDecayStart
	)

parameters AreaLight rollout:AreaLightAdvanced_roll (
	areaJitter			type:#float		default:0			animatable:false 	ui:areaJitter
	)

rollout AreaLight_roll "Area Light Parameters" (
	colorpicker lightColor		"Light Color:"			color:[255,255,255]
	checkbox visibleOn			"Renderable"			checked:true			type:#boolean
	spinner areaIntensity		"Multiplier:" 			range:[0,99999,3] 		scale:0.01		type:#float
	spinner	lightsHotspot		"HotSpot Deg:"			range:[0,179,90]		scale:1			type:#integer
	spinner lightsFalloff		"Falloff Deg:"			range:[0,179,110]	 	scale:1			type:#integer
	group "Shadow Properties" (
		checkbox areaShadows		"Cast Shadows"		checked:true			type:#boolean
		checkbox areaShadowsRTd		"Raytrace Shadows"	checked:false			type:#boolean
		spinner areaShadowMapSize	"Map Size"			range:[0,2048,256]		scale:256		type:#integer
		spinner areaShadowMapSample	"Sample Range"		range:[0.1,16,4]		scale:0.1		type:#float
		spinner areaShadowMapBias	"Bias"				range:[0.01,2,1]		scale:0.01		type:#float
		)
	group "General" (
		checkbox areaOvershoot		"Overshoot"			checked:true			type:#boolean
		checkbox lightsSpecular		"Affect Specular"	checked:true			type:#boolean
		checkbox arealightOn		"On"				checked:true			type:#boolean
		)
	)

rollout AreaLightFalloff_roll "Light Decay" (
	group "Attenuation Params" (
		dropdownlist 	lightsDecayType		"Type:"		items:#("None","Inverse","Inverse Square")
		spinner			lightsDecayStart	"Start:"	range:[0,9999,40]		scale:1			type:#integer
		)
	)
	
rollout AreaLightAdvanced_roll "Advanced Control" (
	group "Jitter" (
		spinner			areaJitter			"Jitter:"		range:[0,9999,0]		scale:0.1			type:#float
		)
	)


---
on create do (
	callbacks.removescripts id:#AreaLight

	toExecute = "	EscapeEnable = False	\n"
	toExecute += "	goRender = True		\n"
		
	toExecute += "	for PlanGomtrieXX in $arealight* where PlanGomtrieXX.arealightOn == true do (	\n"
	toExecute += "		PlanGomtrieXX.renderable = PlanGomtrieXX.visibleOn		\n"
	toExecute += "		if ((getnumverts PlanGomtrieXX.mesh) > 100 ) then ( 	\n"
	toExecute += "			goRender = False 	\n"
	toExecute += "			if ((getnumverts PlanGomtrieXX.mesh) > 400 ) then ( 	\n"
	toExecute += "				if (queryBox \"WARNING: VERY Large amount of lights in arealight. Your system might become unstable. Are you sure you want to continue?\" == True)  then  ( goRender = True )		\n"
	toExecute += "				)	\n"
	toExecute += "			else (			\n"
	toExecute += "				if (queryBox \"WARNING: Large amount of lights in arealight. Do you want to continue?\" == True)  then  ( goRender = True )		\n"
	toExecute += "				)	\n"
	toExecute += "			)	\n"

	toExecute += "	if (goRender == True) then (	\n"
	toExecute += "		if (PlanGomtrieXX.material == undefined) then (		\n"
	toExecute += "			arealightMat = standardMaterial diffuse:(color 255 255 255) name:\"arealightMat\" selfIllumAmount:100		\n"
	toExecute += "			PlanGomtrieXX.material = arealightMat 		\n"
	toExecute += "			)		\n"
	toExecute += "		else ( arealightMat = PlanGomtrieXX.material  )		\n"

	toExecute += "		addModifierToALight = False			 \n"
	toExecute += "		if (PlanGomtrieXX.material.diffuseMap != undefined) then (		\n"
	toExecute += "			if (PlanGomtrieXX.modifiers.count == 0) then (  			\n"
	toExecute += "				addModifierToALight = True  					\n"
	toExecute += "				)												\n"
	toExecute += "			else if (PlanGomtrieXX.modifiers.count > 0) then (					\n"
	toExecute += "				for j = 1 to PlanGomtrieXX.modifiers.count do ( 				\n"
	toExecute += "					addModifierToALight = True			 \n"
	toExecute += "					if (classof PlanGomtrieXX.modifiers[j] == Uvwmap) then ( 			\n"
	toExecute += "						addModifierToALight = False							 \n"
	toExecute += "						exit							 \n"
	toExecute += "						)													 \n"
	toExecute += "					)		\n"
	toExecute += "				)		\n"
	toExecute += "			)			\n"
	toExecute += "		if (addModifierToALight == True) then (		\n"
	toExecute += "			addmodifier PlanGomtrieXX (uvwmap())  					\n"
	toExecute += "			)			\n"

	toExecute += "		if (PlanGomtrieXX.visibleOn == True) then (		\n"
	toExecute += "			arealightMat.diffuse = PlanGomtrieXX.lightColor		\n"
	toExecute += "			PlanGomtrieXX.material = arealightMat		\n"
	toExecute += "			)		\n"

	toExecute += "			if (PlanGomtrieXX.material.diffuseMap != undefined) then (		\n"
	toExecute += "				areaBitmapW = PlanGomtrieXX.material.diffuseMap.bitmap.width		\n"
	toExecute += "				areaBitmapH = PlanGomtrieXX.material.diffuseMap.bitmap.height		\n"
	toExecute += "				callbackBitmap = renderMap PlanGomtrieXX.material.diffuseMap size:[areaBitmapW, areaBitmapH]		\n"
	toExecute += "				)		\n"

	toExecute += "		PlanGomtrieXX.castshadows = false 	\n"

	toExecute += "		for i = 1 to PlanGomtrieXX.mesh.numverts do (		\n"
	toExecute += "			allTempLights = undefined	\n"
	toExecute += "			lighttransform = rotateXMatrix 180 * PlanGomtrieXX.objecttransform	 \n"

	toExecute += "			areaRandomJitter = random 0 PlanGomtrieXX.areaJitter 	 \n"

	toExecute += "			allTempLights = freeSpot pos:([(getVert PlanGomtrieXX.mesh i).x+areaRandomJitter,(getVert PlanGomtrieXX.mesh i).y+areaRandomJitter,(getVert PlanGomtrieXX.mesh i).z+1] * PlanGomtrieXX.objecttransform) transform:(lighttransform)	\n"
	toExecute += "			allTempLights.name = uniquename \"targeted_arealight_\"		\n"	
	toExecute += "			allTempLights.multiplier = PlanGomtrieXX.areaIntensity / PlanGomtrieXX.mesh.numverts	\n"

	toExecute += "			if (PlanGomtrieXX.material.diffuseMap != undefined) then (		\n"
	toExecute += "				bitmapColoratX = (abs(mod(getTVert PlanGomtrieXX.mesh i).x 1)*areaBitmapW) as integer			\n"
	toExecute += "				bitmapColoratY = (areaBitmapH - (abs(mod(getTVert PlanGomtrieXX.mesh i).y 1)*areaBitmapH)) as integer 			\n"
	toExecute += "				lightColor = (getpixels callbackBitmap [bitmapColoratX ,bitmapColoratY] 1)[1]			\n"
	toExecute += "				)		\n"
	toExecute += "			else ( lightColor = PlanGomtrieXX.lightColor	)		\n"
	
	toExecute += "			allTempLights.rgb = lightColor 			\n"
	toExecute += "			allTempLights.hotspot = PlanGomtrieXX.lightsHotspot			\n"
	toExecute += "			allTempLights.falloff = PlanGomtrieXX.lightsFalloff			\n"
	toExecute += "				if (PlanGomtrieXX.lightsDecayType == 3) then ( allTempLights.attenDecay = 3	)		\n"
	toExecute += "				else if (PlanGomtrieXX.lightsDecayType == 2) then ( allTempLights.attenDecay = 2 )			\n"
	toExecute += "				else if (PlanGomtrieXX.lightsDecayType == 1) then ( allTempLights.attenDecay = 1 )		\n"
	toExecute += "			allTempLights.DecayRadius = PlanGomtrieXX.lightsDecayStart			\n"
	toExecute += "			if (PlanGomtrieXX.areaShadowsRTd == true) then ( \n"
	toExecute += "					allTempLights.raytracedShadows = PlanGomtrieXX.areaShadowsRTd		\n"
	toExecute += "				) 	else (	\n"
	toExecute += "					allTempLights.mapsize = PlanGomtrieXX.areaShadowMapSize				\n"
	toExecute += "					allTempLights.samplerange = PlanGomtrieXX.areaShadowMapSample		\n"
	toExecute += "					allTempLights.mapbias = PlanGomtrieXX.areaShadowMapBias				\n"
	toExecute += "				) 	\n"

	toExecute += "			allTempLights.baseobject.castshadows = PlanGomtrieXX.areaShadows		\n"	
	toExecute += "			allTempLights.overshoot = PlanGomtrieXX.areaOvershoot			\n"	
	toExecute += "			allTempLights.affectspecular = PlanGomtrieXX.lightsSpecular		\n"	

	toExecute += "		)\n"
	toExecute += "	)\n"
	toExecute += "	)\n"

	callbacks.addScript #preRender toExecute id:#AreaLight persistent:true

-- Cleanup Phase
	toExecuteEnd = "	delete $targeted_arealight*	\n"
	toExecuteEnd += "	freeSceneBitmaps		\n"
	toExecuteEnd += "	EscapeEnable = true		\n"
	callbacks.addScript #postRender toExecuteEnd id:#AreaLight persistent:true

	) 


) -- END

